var AWS = require('aws-sdk'),
    lambda = new AWS.Lambda(),
    apigateway = new AWS.APIGateway();

/**
 * It is a built-in module developed by AWS. It is used to send failure/success signals to Cloudformation engine.
 * For more information about the module, see:
 * https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-code.html#cfn-lambda-function-code-cfnresponsemodule
 */
var response = require('cfn-response');

exports.handler = (event, context) => {

    /**
     * In case of Delete signal, we do not have to do nothing because this script is not creating any persistent resource.
     * We can directly return with success signal.
     */
    if (event.RequestType == 'Delete') {
        return response.send(event, context, response.SUCCESS);
    }

    switch (event.ResourceType) {
        /**
         * A LambdaVersion is being created.
         * Here we invoke the Lambda API to publish a new version of the Lambda.
         */
        case 'Custom::LambdaVersion':
            var lambdaName = (event.ResourceProperties.FunctionName !== 'undefined' ? event.ResourceProperties.FunctionName : null);
            lambda.publishVersion({FunctionName: lambdaName}).promise().then((data) => {
                return response.send(event, context, response.SUCCESS, {'Version': data.Version}, data.FunctionArn);
            }).catch((err)=> {
                return response.send(event, context, response.FAILED, err);
            });
            break;
        /**
         * A Api Gateway Deployment is being created.
         * Here we invoke the API Gateway Management API to publish a new version of the API Gateway REST API.
         */
        case 'Custom::ApiDeployment':
            var restApiId = (event.ResourceProperties.RestApiId !== 'undefined' ? event.ResourceProperties.RestApiId : null);
            var stageName = (event.ResourceProperties.StageName !== 'undefined' ? event.ResourceProperties.StageName : null);
            apigateway.createDeployment({restApiId: restApiId, stageName: stageName}).promise().then((data) => {
                return response.send(event, context, response.SUCCESS, null, data.id);
            }).catch((err)=> {
                return response.send(event, context, response.FAILED, err);
            });
            break;
        default:
            return response.send(event, context, response.FAILED, 'This function only supports Custom::LambdaVersion and Custom::ApiDeployment resource types.');
    }
};
